@@ -42,6 +42,7 @@
 
 #include <stdlib.h>
 
+#include "gioalias.h"
 
 /**
  * SECTION:gresolver
@@ -758,13 +759,13 @@ _g_resolver_addresses_from_addrinfo (const char       *hostname,
 void
 _g_resolver_address_to_sockaddr (GInetAddress            *address,
                                  struct sockaddr_storage *sa,
-                                 gsize                   *len)
+                                 gsize                   *sa_len)
 {
   GSocketAddress *sockaddr;
 
   sockaddr = g_inet_socket_address_new (address, 0);
   g_socket_address_to_native (sockaddr, (struct sockaddr *)sa, sizeof (*sa), NULL);
-  *len = g_socket_address_get_native_size (sockaddr);
+  *sa_len = g_socket_address_get_native_size (sockaddr);
   g_object_unref (sockaddr);
 }
 
@@ -929,3 +930,6 @@ _g_resolver_targets_from_DnsQuery (const gchar  *rrname,
 }
 
 #endif
+
+#define __G_RESOLVER_C__
+#include "gioaliasdef.c"
