@@ -38,7 +38,9 @@
 # include <unistd.h>
 #endif
 
+#ifdef HAVE_SYS_UIO_H
 #include <sys/uio.h>
+#endif
 
 #include "gsocket.h"
 #include "gcancellable.h"
@@ -1330,6 +1332,7 @@ g_socket_speaks_ipv4 (GSocket *socket)
 /**
  * g_socket_accept:
  * @socket: a #GSocket.
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Accept incoming connections on a connection-based socket. This removes
@@ -1350,6 +1353,7 @@ g_socket_speaks_ipv4 (GSocket *socket)
  */
 GSocket *
 g_socket_accept (GSocket       *socket,
+		 GCancellable  *cancellable,
 		 GError       **error)
 {
   GSocket *new_socket;
@@ -1364,7 +1368,7 @@ g_socket_accept (GSocket       *socket,
     {
       if (socket->priv->blocking &&
 	  !g_socket_condition_wait (socket,
-				    G_IO_IN, NULL, error))
+				    G_IO_IN, cancellable, error))
 	return NULL;
 
       if ((ret = accept (socket->priv->fd, NULL, 0)) < 0)
@@ -1440,6 +1444,7 @@ g_socket_accept (GSocket       *socket,
  * g_socket_connect:
  * @socket: a #GSocket.
  * @address: a #GSocketAddress specifying the remote address.
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Connect the socket to the specified remote address.
@@ -1466,6 +1471,7 @@ g_socket_accept (GSocket       *socket,
 gboolean
 g_socket_connect (GSocket         *socket,
 		  GSocketAddress  *address,
+		  GCancellable    *cancellable,
 		  GError         **error)
 {
   struct sockaddr_storage buffer;
@@ -1496,11 +1502,12 @@ g_socket_connect (GSocket         *socket,
 	    {
 	      if (socket->priv->blocking)
 		{
-		  g_socket_condition_wait (socket, G_IO_OUT, NULL, NULL);
-		  if (g_socket_check_connect_result (socket, error))
-		    break;
-		  else
-		    g_prefix_error (error, _("Error connecting: "));
+		  if (g_socket_condition_wait (socket, G_IO_OUT, cancellable, error))
+		    {
+		      if (g_socket_check_connect_result (socket, error))
+			break;
+		    }
+		  g_prefix_error (error, _("Error connecting: "));
 		}
 	      else
                 g_set_error_literal (error, G_IO_ERROR, G_IO_ERROR_PENDING,
@@ -1568,6 +1575,7 @@ g_socket_check_connect_result (GSocket  *socket,
  * @buffer: a buffer to read data into (which should be at least @size
  *     bytes long).
  * @size: the number of bytes you want to read from the socket
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Receive data (up to @size bytes) from a socket. This is mainly used by
@@ -1598,10 +1606,11 @@ g_socket_check_connect_result (GSocket  *socket,
  * Since: 2.22
  */
 gssize
-g_socket_receive (GSocket  *socket,
-		  gchar    *buffer,
-		  gsize     size,
-		  GError  **error)
+g_socket_receive (GSocket       *socket,
+		  gchar         *buffer,
+		  gsize          size,
+		  GCancellable  *cancellable,
+		  GError       **error)
 {
   gssize ret;
 
@@ -1610,11 +1619,14 @@ g_socket_receive (GSocket  *socket,
   if (!check_socket (socket, error))
     return -1;
 
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    return -1;
+
   while (1)
     {
       if (socket->priv->blocking &&
 	  !g_socket_condition_wait (socket,
-				    G_IO_IN, NULL, error))
+				    G_IO_IN, cancellable, error))
 	return -1;
 
       if ((ret = recv (socket->priv->fd, buffer, size, 0)) < 0)
@@ -1659,6 +1671,7 @@ g_socket_receive (GSocket  *socket,
  * @buffer: a buffer to read data into (which should be at least @size
  *     bytes long).
  * @size: the number of bytes you want to read from the socket
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Receive data (up to @size bytes) from a socket.
@@ -1678,6 +1691,7 @@ g_socket_receive_from (GSocket         *socket,
 		       GSocketAddress **address,
 		       gchar           *buffer,
 		       gsize            size,
+		       GCancellable    *cancellable,
 		       GError         **error)
 {
   GInputVector v;
@@ -1689,14 +1703,26 @@ g_socket_receive_from (GSocket         *socket,
 				   address,
 				   &v, 1,
 				   NULL, 0, NULL,
+				   cancellable,
 				   error);
 }
 
+/* Although we ignore SIGPIPE, gdb will still stop if the app receives
+ * one, which can be confusing and annoying. So if possible, we want
+ * to suppress the signal entirely.
+ */
+#ifdef MSG_NOSIGNAL
+#define G_SOCKET_DEFAULT_SEND_FLAGS MSG_NOSIGNAL
+#else
+#define G_SOCKET_DEFAULT_SEND_FLAGS 0
+#endif
+
 /**
  * g_socket_send:
  * @socket: a #GSocket
  * @buffer: the buffer containing the data to send.
  * @size: the number of bytes to send
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Tries to send @size bytes from @buffer on the socket. This is
@@ -1720,10 +1746,11 @@ g_socket_receive_from (GSocket         *socket,
  * Since: 2.22
  */
 gssize
-g_socket_send (GSocket      *socket,
-	       const gchar  *buffer,
-	       gsize         size,
-	       GError      **error)
+g_socket_send (GSocket       *socket,
+	       const gchar   *buffer,
+	       gsize          size,
+	       GCancellable  *cancellable,
+	       GError       **error)
 {
   gssize ret;
 
@@ -1732,14 +1759,17 @@ g_socket_send (GSocket      *socket,
   if (!check_socket (socket, error))
     return -1;
 
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    return -1;
+
   while (1)
     {
       if (socket->priv->blocking &&
 	  !g_socket_condition_wait (socket,
-				    G_IO_OUT, NULL, error))
+				    G_IO_OUT, cancellable, error))
 	return -1;
 
-      if ((ret = send (socket->priv->fd, buffer, size, 0)) < 0)
+      if ((ret = send (socket->priv->fd, buffer, size, G_SOCKET_DEFAULT_SEND_FLAGS)) < 0)
 	{
 	  int errsv = get_socket_errno ();
 
@@ -1780,6 +1810,7 @@ g_socket_send (GSocket      *socket,
  * @address: a #GSocketAddress, or %NULL
  * @buffer: the buffer containing the data to send.
  * @size: the number of bytes to send
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Tries to send @size bytes from @buffer to @address. If @address is
@@ -1798,6 +1829,7 @@ g_socket_send_to (GSocket         *socket,
 		  GSocketAddress  *address,
 		  const gchar     *buffer,
 		  gsize            size,
+		  GCancellable    *cancellable,
 		  GError         **error)
 {
   GOutputVector v;
@@ -1809,7 +1841,9 @@ g_socket_send_to (GSocket         *socket,
 				address,
 				&v, 1,
 				NULL, 0,
-				0, error);
+				0,
+				cancellable,
+				error);
 }
 
 /**
@@ -2231,7 +2265,10 @@ winsock_finalize (GSource *source)
   g_object_unref (socket);
 
   if (winsock_source->cancellable)
-    g_object_unref (winsock_source->cancellable);
+    {
+      g_cancellable_release_fd (winsock_source->cancellable);
+      g_object_unref (winsock_source->cancellable);
+    }
 }
 
 static GSourceFuncs winsock_funcs =
@@ -2267,11 +2304,10 @@ winsock_source_new (GSocket      *socket,
   winsock_source->condition = condition;
   add_condition_watch (socket, &winsock_source->condition);
 
-  if (cancellable)
+  if (g_cancellable_make_pollfd (cancellable,
+                                 &winsock_source->cancel_pollfd))
     {
       winsock_source->cancellable = g_object_ref (cancellable);
-      g_cancellable_make_pollfd (cancellable,
-				 &winsock_source->cancel_pollfd);
       g_source_add_poll (source, &winsock_source->cancel_pollfd);
     }
 
@@ -2422,11 +2458,8 @@ g_socket_condition_wait (GSocket       *socket,
     num_events = 0;
     events[num_events++] = socket->priv->event;
 
-    if (cancellable)
-      {
-	g_cancellable_make_pollfd (cancellable, &cancel_fd);
-	events[num_events++] = (WSAEVENT)cancel_fd.fd;
-      }
+    if (g_cancellable_make_pollfd (cancellable, &cancel_fd))
+      events[num_events++] = (WSAEVENT)cancel_fd.fd;
 
     current_condition = update_condition (socket);
     while ((condition & current_condition) == 0)
@@ -2450,6 +2483,8 @@ g_socket_condition_wait (GSocket       *socket,
 	current_condition = update_condition (socket);
       }
     remove_condition_watch (socket, &condition);
+    if (num_events > 1)
+      g_cancellable_release_fd (cancellable);
 
     return (condition & current_condition) != 0;
   }
@@ -2463,15 +2498,15 @@ g_socket_condition_wait (GSocket       *socket,
     poll_fd[0].events = condition;
     num = 1;
 
-    if (cancellable)
-      {
-	g_cancellable_make_pollfd (cancellable, &poll_fd[1]);
-	num++;
-      }
+    if (g_cancellable_make_pollfd (cancellable, &poll_fd[1]))
+      num++;
 
     do
       result = g_poll (poll_fd, num, -1);
     while (result == -1 && get_socket_errno () == EINTR);
+    
+    if (num > 1)
+      g_cancellable_release_fd (cancellable);
 
     return cancellable == NULL ||
       !g_cancellable_set_error_if_cancelled (cancellable, error);
@@ -2489,6 +2524,7 @@ g_socket_condition_wait (GSocket       *socket,
  *   %NULL.
  * @num_messages: number of elements in @messages, or -1.
  * @flags: an int containing #GSocketMsgFlags flags
+ * @cancellable: a %GCancellable or %NULL
  * @error: #GError for error reporting, or %NULL to ignore.
  *
  * Send data to @address on @socket.  This is the most complicated and
@@ -2542,6 +2578,7 @@ g_socket_send_message (GSocket                *socket,
 		       GSocketControlMessage **messages,
 		       gint                    num_messages,
 		       gint                    flags,
+		       GCancellable           *cancellable,
 		       GError                **error)
 {
   GOutputVector one_vector;
@@ -2550,6 +2587,9 @@ g_socket_send_message (GSocket                *socket,
   if (!check_socket (socket, error))
     return -1;
 
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    return -1;
+
   if (num_vectors == -1)
     {
       for (num_vectors = 0;
@@ -2589,6 +2629,11 @@ g_socket_send_message (GSocket                *socket,
 	if (!g_socket_address_to_native (address, msg.msg_name, msg.msg_namelen, error))
 	  return -1;
       }
+    else
+      {
+	msg.msg_name = NULL;
+	msg.msg_namelen = 0;
+      }
 
     /* iov */
     {
@@ -2648,10 +2693,10 @@ g_socket_send_message (GSocket                *socket,
       {
 	if (socket->priv->blocking &&
 	    !g_socket_condition_wait (socket,
-				      G_IO_OUT, NULL, error))
+				      G_IO_OUT, cancellable, error))
 	  return -1;
 
-	result = sendmsg (socket->priv->fd, &msg, flags);
+	result = sendmsg (socket->priv->fd, &msg, flags | G_SOCKET_DEFAULT_SEND_FLAGS);
 	if (result < 0)
 	  {
 	    int errsv = get_socket_errno ();
@@ -2716,7 +2761,7 @@ g_socket_send_message (GSocket                *socket,
       {
 	if (socket->priv->blocking &&
 	    !g_socket_condition_wait (socket,
-				      G_IO_OUT, NULL, error))
+				      G_IO_OUT, cancellable, error))
 	  return -1;
 
 	if (address)
@@ -2770,6 +2815,7 @@ g_socket_send_message (GSocket                *socket,
  * @num_messages: a pointer which will be filled with the number of
  *    elements in @messages, or %NULL
  * @flags: a pointer to an int containing #GSocketMsgFlags flags
+ * @cancellable: a %GCancellable or %NULL
  * @error: a #GError pointer, or %NULL
  *
  * Receive data from a socket.  This is the most complicated and
@@ -2840,6 +2886,7 @@ g_socket_receive_message (GSocket                 *socket,
 			  GSocketControlMessage ***messages,
 			  gint                    *num_messages,
 			  gint                    *flags,
+			  GCancellable            *cancellable,
 			  GError                 **error)
 {
   GInputVector one_vector;
@@ -2848,6 +2895,9 @@ g_socket_receive_message (GSocket                 *socket,
   if (!check_socket (socket, error))
     return -1;
 
+  if (g_cancellable_set_error_if_cancelled (cancellable, error))
+    return -1;
+
   if (num_vectors == -1)
     {
       for (num_vectors = 0;
@@ -2925,7 +2975,7 @@ g_socket_receive_message (GSocket                 *socket,
       {
 	if (socket->priv->blocking &&
 	    !g_socket_condition_wait (socket,
-				      G_IO_IN, NULL, error))
+				      G_IO_IN, cancellable, error))
 	  return -1;
 
 	result = recvmsg (socket->priv->fd, &msg, msg.msg_flags);
@@ -3053,7 +3103,7 @@ g_socket_receive_message (GSocket                 *socket,
       {
 	if (socket->priv->blocking &&
 	    !g_socket_condition_wait (socket,
-				      G_IO_IN, NULL, error))
+				      G_IO_IN, cancellable, error))
 	  return -1;
 
 	addrlen = sizeof addr;
